#version 330 core

layout (triangles) in;
layout (triangle_strip, max_vertices = 3) out;

in VertexData {
    flat uint objectId;
    vec3 Normal;
    vec4 color;
    vec3 FragPos;
    vec2 texcoord;
    vec4 FragPosLightSpace0;
    vec4 FragPosLightSpace1;
    vec4 FragPosLightSpace2;
    vec4 VertexPosition;
} inData[];

out VertexData {
    flat uint objectId;
    vec3 Normal;
    vec4 color;
    vec3 FragPos;
    vec2 texcoord;
    vec4 FragPosLightSpace0;
    vec4 FragPosLightSpace1;
    vec4 FragPosLightSpace2;
    vec4 VertexPosition;
} outData;

uniform bool u_alpha = false;
uniform usamplerBuffer u_visibility;
uniform samplerBuffer u_colors;

void main()
{
    vec4 objectColor = texelFetch(u_colors, int(inData[0].objectId));

    // uniform ifs can be optimized away?
    if (u_alpha)
    {
        if (objectColor.a == 1)
        {
            return;
        }
    }
    else
    {
        if (objectColor.a != 1)
        {
            return;
        }
    }

    uvec4 visible = texelFetch(u_visibility, int(inData[0].objectId));

    if (visible.r == 0u) {
        return;
    }

    for (int i = 0; i < 3; ++i) {
        outData.objectId = inData[i].objectId;
        outData.Normal = inData[i].Normal;
        outData.color = inData[i].color;
        outData.texcoord = inData[i].texcoord;
        outData.FragPosLightSpace0 = inData[i].FragPosLightSpace0;
        outData.FragPosLightSpace1 = inData[i].FragPosLightSpace1;
        outData.FragPosLightSpace2 = inData[i].FragPosLightSpace2;
        outData.VertexPosition = inData[i].VertexPosition;

        gl_Position = gl_in[i].gl_Position;
        EmitVertex();
    }

    EndPrimitive();
}
